#State 0 comes on.  Inhibits State 1 until it comes on, in spinnaker.

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle
 
import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 100
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createTwoInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [52.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')

    return [spikeGen0,spikeGen1]

def createNeurons():
    if (nealParameters.simulator == 'nest'):
            numNeurons = fsa.CA_SIZE * 5
    elif (nealParameters.simulator == 'spinnaker'):
            numNeurons = 100
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def setupRecording(cells):
    if  (nealParameters.simulator == 'nest'):
        cells.record({'spikes','v'})
    elif  (nealParameters.simulator == 'spinnaker'):
        cells.record()

def printResults(spinnCells):
    #print
    if simulator == "nest":
        spikes = nest.GetStatus(nestRecorder)
        test = spikes[0]['events']['times']
        print 'spikes', test
    elif simulator == 'spinnaker':
        spinnCells.printSpikes('temp.sp')


def stateInhibitsState(fromCA,toCA):
    if (nealParameters.simulator == "spinnaker"):
        preventWeight = 0.3
    elif (nealParameters.simulator=="nest"):
        preventWeight = -0.3
    connector = []
    for fromOffset in range (0,fsa.CA_SIZE):
            fromNeuron = fromOffset + (fromCA*fsa.CA_SIZE)
            for toOffset in range (0,fsa.CA_SIZE):
                toNeuron = toOffset + (toCA*fsa.CA_SIZE)
                connector = connector + [(fromNeuron,toNeuron,preventWeight,
                                      nealParameters.DELAY)]
    neal.nealProjection(stateCells,stateCells,connector,
                                 'inhibitory')


##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)

init()

spikeGenerators = createTwoInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]

stateCells = createNeurons()

setupRecording(stateCells)


#Build the FSA
fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,stateCells,0)
fsa.turnOnStateFromSpikeSource(secondSpikeGenerator,stateCells,2)
fsa.makeCA(stateCells,0)
fsa.makeCA(stateCells,1)
fsa.makeCA(stateCells,2)

fsa.stateTurnsOffState(stateCells,0,stateCells,1)
fsa.stateTurnsOffState(stateCells,2,stateCells,1)

sim.run(RUN_DURATION)

fsa.testPrintResults(stateCells)



