#This makes 1 state, and uses a spike source to turn it on, stimulate it,
#inhibit it, and then shut it off.

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 300
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)


def printPklSpikes(fileName):
    fileHandle = open(fileName)
    neoObj = pickle.load(fileHandle)
    segments = neoObj.segments
    segment = segments[0]
    spikeTrains = segment.spiketrains
    neurons = len(spikeTrains)
    for neuronNum in range (0,neurons):
        if (len(spikeTrains[neuronNum])>0):
            spikes = spikeTrains[neuronNum]
            for spike in range (0,len(spikes)):
                print neuronNum, spikes[spike]
    fileHandle.close()
    

def createFourInputs():
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')

    inputSpikeTimes1 = [100.0]
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')

    inputSpikeTimes2 = [150.0]
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')

    inputSpikeTimes3 = [200.0]
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3]


def createNeurons():
    if (nealParameters.simulator == 'nest'):
        numNeurons = fsa.CA_SIZE * 2
    elif (nealParameters.simulator == 'spinnaker'):
        numNeurons = 100
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def setupRecording(cells):
    if (nealParameters.simulator == 'nest'):
        cells.record({'spikes','v'})
    elif  (nealParameters.simulator == 'spinnaker'):
        cells.record()
        cells.record_v()

def printResults(simCells):
    #print
    if  (nealParameters.simulator == 'nest'):
        simCells.printSpikes('temp.pkl')
        #printPklSpikes('temp.pkl')
    elif (nealParameters.simulator == 'spinnaker'):
        simCells.printSpikes('temp.sp')
        simCells.print_v('temp.volts')

##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)
init()
spikeGenerators =  createFourInputs()

stateCells = createNeurons()

setupRecording(stateCells)

#make a state
fsa.makeCA(stateCells,0)

#play with the state using spike sources
fsa.turnOnStateFromSpikeSource(spikeGenerators[0],stateCells,0)
fsa.stimulateStateFromSpikeSource(spikeGenerators[1],stateCells,0,0.1)
fsa.inhibitStateFromSpikeSource(spikeGenerators[2],stateCells,0,-0.2)
fsa.turnOffStateFromSpikeSource(spikeGenerators[3],stateCells,0)

sim.run(RUN_DURATION)

printResults(stateCells)
