#This is a two state system.  The spike sources turn them both on.
#The second inhibits the first, so it runs slower.

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 200

def createTwoInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [100.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    return [spikeGen0,spikeGen1]

def createNeurons():
    if (nealParameters.simulator == 'nest'):
            numNeurons = fsa.CA_SIZE * 2
    elif (nealParameters.simulator == 'spinnaker'):
            numNeurons = 100
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)

    return cells


##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)

fsa.testInit()

#setup input
spikeGenerators = createTwoInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]

#make neurons and make them recordable
stateCells = createNeurons()
fsa.testSetupRecording(stateCells)

#build the fsa and run it
#Build the FSA 
fsa.makeCA(stateCells,0)
fsa.makeCA(stateCells,1)
fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,stateCells,0)
fsa.turnOnStateFromSpikeSource(secondSpikeGenerator,stateCells,1)

if (nealParameters.simulator == 'nest'):
    fsa.stateInhibitsState(stateCells,1,stateCells,0, -0.02)
elif (nealParameters.simulator == 'spinnaker'):
    fsa.stateInhibitsState(stateCells,1,stateCells,0, 0.02)

#run the simulator
fsa.testRunFSA(RUN_DURATION)

#print results
fsa.testPrintResults(stateCells)



