#This makes 1 state, and turns it on.  This then interacts with other neurons
#to test the three stateOneNeuron functions.


import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 300
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)


def printPklSpikes(fileName):
    fileHandle = open(fileName)
    neoObj = pickle.load(fileHandle)
    segments = neoObj.segments
    segment = segments[0]
    spikeTrains = segment.spiketrains
    neurons = len(spikeTrains)
    for neuronNum in range (0,neurons):
        if (len(spikeTrains[neuronNum])>0):
            spikes = spikeTrains[neuronNum]
            for spike in range (0,len(spikes)):
                print neuronNum, spikes[spike]
    fileHandle.close()
    

def createOneInput():
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')

    return spikeGen0

def createNeurons():
    if (nealParameters.simulator == 'nest'):
        numNeurons = fsa.CA_SIZE * 2
    elif (nealParameters.simulator == 'spinnaker'):
        numNeurons = 100
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def setupRecording(cells):
    if (nealParameters.simulator == 'nest'):
        cells.record({'spikes','v'})
    elif  (nealParameters.simulator == 'spinnaker'):
        cells.record()
        cells.record_v()

def printResults(simCells):
    #print
    if  (nealParameters.simulator == 'nest'):
        simCells.printSpikes('temp.pkl')
        #printPklSpikes('temp.pkl')
    elif (nealParameters.simulator == 'spinnaker'):
        simCells.printSpikes('temp.sp')
        simCells.print_v('temp.volts')

def spikeSourceFiresNeuron(spikeSource,cells,neuronNumber):
    connector = [(0,neuronNumber,fsa.INPUT_WEIGHT,nealParameters.DELAY)]
    neal.nealProjection(spikeSource,cells, connector,'excitatory')


##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)
init()
firstSpikeGenerator =  createOneInput()

stateCells = createNeurons()

setupRecording(stateCells)

#make a state
fsa.makeCA(stateCells,0)

#play with the state using spike sources
fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,stateCells,0)
fsa.stateTurnsOnOneNeuron(stateCells,0,stateCells,10)
fsa.stateStimulatesOneNeuron(stateCells,0,stateCells,11,0.004)

fsa.stateInhibitsOneNeuron(stateCells,0,stateCells,0,-0.004) 

sim.run(RUN_DURATION)

printResults(stateCells)
